==> api-0.6.0-sources.jar: threads/core/ConnectService.java <==
package threads.core;

import android.content.Context;
import android.util.Log;

import androidx.annotation.NonNull;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;

import threads.core.api.Addresses;
import threads.core.api.Members;
import threads.core.api.MultiAddresses;
import threads.core.api.Note;
import threads.core.api.Thread;
import threads.ipfs.IPFS;
import threads.ipfs.api.PID;

import static androidx.core.util.Preconditions.checkArgument;
import static androidx.core.util.Preconditions.checkNotNull;

public class ConnectService {

    private static final String TAG = ConnectService.class.getSimpleName();

    public static boolean connectPeer(@NonNull Context context,
                                      @NonNull PID peer,
                                      @NonNull String aesKey,
                                      boolean supportDiscovery,
                                      boolean protectRelay,
                                      boolean protectPeer,
                                      boolean updateUser) {
        checkNotNull(context);
        checkNotNull(aesKey);
        checkNotNull(peer);
        final int timeout = Preferences.getConnectionTimeout(context);
        return connectPeer(context, peer, aesKey, timeout,
                supportDiscovery, protectRelay, protectPeer, updateUser);
    }

    public static boolean connectPeer(@NonNull Context context,
                                      @NonNull PID pid,
                                      @NonNull String aesKey,
                                      int timeout,
                                      boolean supportDiscovery,
                                      boolean protectRelay,
                                      boolean protectPeer,
                                      boolean updateUser) {
        checkNotNull(context);
        checkNotNull(aesKey);
        checkNotNull(pid);
        checkArgument(timeout > 0);
        if (!Network.isConnected(context)) {
            return false;
        }


        if (supportDiscovery) {
            threads.core.api.PeerInfo peer = IdentityService.getPeerInfo(
                    context, pid, aesKey, updateUser);
            if (peer != null) {

                if (swarmConnect(context, peer, timeout, protectRelay, protectPeer)) {
                    return true;
                }

            }

        }
        final IPFS ipfs = Singleton.getInstance(context).getIpfs();
        if (ipfs != null) {
            boolean connected = ipfs.swarmConnect(pid, timeout);
            if (connected) {
                if (protectPeer) {
                    ipfs.protectPeer(pid, GatewayService.TAG);
                }
            }
            return ipfs.isConnected(pid);

        }
        return false;
    }


    public static void swarmDisconnect(@NonNull Context context,
                                       @NonNull threads.core.api.PeerInfo peer,
                                       boolean unprotectPeer){
        checkNotNull(peer);
        final IPFS ipfs = Singleton.getInstance(context).getIpfs();
        if (ipfs != null) {
            Addresses addresses = peer.getAddresses();
            for (String relay : addresses.keySet()) {
                PID pid = PID.create(relay);
                if(unprotectPeer) {
                    ipfs.unProtectPeer(pid, GatewayService.TAG); // TODO check if unprotect, then close conn
                }
                ipfs.swarmDisconnect(pid);
            }
        }
    }

    public static boolean swarmConnect(@NonNull Context context,
                                       @NonNull threads.core.api.PeerInfo peer,
                                       int timeout,
                                       boolean protectRelay,
                                       boolean protectPeer) {
        checkNotNull(context);
        checkNotNull(peer);
        checkArgument(timeout > 0);

        final Singleton.ConsoleListener consoleListener =
                Singleton.getInstance(context).getConsoleListener();

        final IPFS ipfs = Singleton.getInstance(context).getIpfs();
        if (ipfs != null) {
            Addresses addresses = peer.getAddresses();
            for (String relay : addresses.keySet()) {
                try {
                    String ma = addresses.get(relay);
                    checkNotNull(ma);
                    PID relayPID = PID.create(relay);
                    boolean relayConnected = ipfs.isConnected(relayPID);
                    if (!relayConnected) {
                        relayConnected = ipfs.swarmConnect(
                                ma + "/" + IPFS.Style.p2p.name() + "/" + relay,
                                timeout);
                    }
                    if (protectRelay && relayConnected) {

                        // be pessimistic and assume relay and no autonat and no pubsub
                        // anyway when it is, it they are the will be evaluated later
                        GatewayService.storePeer(context, PID.create(relay), ma,
                                true, false, false, 0);

                        ipfs.protectPeer(PID.create(relay), GatewayService.TAG);
                    }

                    if (relayConnected) {
                        String address = ipfs.relay_address(ma, relayPID, peer.getPID());
                        boolean connect = ipfs.swarmConnect(address, timeout);
                        if (connect) {



                            if (protectPeer) {
                                ipfs.protectPeer(peer.getPID(), GatewayService.TAG);
                            }
                        }
                    }

                } catch (Throwable e) {
                    Log.e(TAG, e.getLocalizedMessage(), e);
                }
            }


            MultiAddresses multiAddresses = peer.getMultiAddresses();
            for (String address : multiAddresses) {

                try {
                    String ma = address.concat(
                            "/" + IPFS.Style.p2p + "/" + peer.getPID().getPid());
                    consoleListener.debug("Try connect : " + ma);


                    if (ipfs.isConnected(peer.getPID())) {
                        if (protectPeer) {
                            ipfs.protectPeer(peer.getPID(), GatewayService.TAG);
                        }
                        break;
                    } else {
                        boolean connect = ipfs.swarmConnect(ma, timeout);

                        if (connect) {

                            if (protectPeer) {
                                ipfs.protectPeer(peer.getPID(), GatewayService.TAG);
                            }

                            consoleListener.info("Success connect : " + ma);
                            break;
                        }
                    }
                } catch (Throwable e) {
                    Log.e(TAG, e.getLocalizedMessage(), e);
                }
            }


            return ipfs.isConnected(peer.getPID());
        }
        return false;
    }

    public static List<Future<Boolean>> connectMembersAsync(
            @NonNull Context context,
            @NonNull threads.core.api.Thread thread,
            @NonNull String aesKey,
            boolean supportDiscovery,
            boolean protectPeer,
            boolean protectRelay,
            boolean updateUser,
            int timeout) {
        checkNotNull(context);
        checkNotNull(thread);
        checkNotNull(aesKey);
        checkArgument(timeout > 0);


        ArrayList<Future<Boolean>> futures = new ArrayList<>();
        ExecutorService executorService = Executors.newFixedThreadPool(5);
        Members members = thread.getMembers();
        for (PID pid : members) {
            Future<Boolean> future = executorService.submit(() -> {
                try {
                    return connectPeer(context, pid, aesKey, timeout,
                            supportDiscovery, protectRelay, protectPeer, updateUser);
                } catch (Throwable e) {
                    Log.e(TAG, e.getLocalizedMessage(), e);
                    return false;
                }
            });
            futures.add(future);
        }
        return futures;
    }


    public static boolean publishNote(@NonNull Context context,
                                      @NonNull Note note,
                                      @NonNull String aesKey) {

        checkNotNull(context);
        checkNotNull(note);
        checkNotNull(aesKey);

        final THREADS threads = Singleton.getInstance(context).getThreads();
        final IPFS ipfs = Singleton.getInstance(context).getIpfs();
        Thread thread = threads.getThread(note);
        checkNotNull(thread);

        boolean notifyAll = true;
        Members members = thread.getMembers();
        for (PID pid : members) {

            try {

                boolean result = ipfs.isConnected(pid);
                if (result) {
                    result = threads.publishNote(ipfs, pid, note);
                }

                if (!result) {
                    notifyAll = false;
                }

            } catch (Throwable e) {
                Log.e(TAG, e.getLocalizedMessage(), e);
                return false;
            }

        }
        return notifyAll;
    }

    public static List<Future<Boolean>> connectMembersAsync(
            @NonNull Context context,
            @NonNull Thread thread,
            @NonNull String aesKey,
            boolean supportDiscovery,
            boolean protectPeer,
            boolean protectRelay,
            boolean updateUser) {

        checkNotNull(context);
        checkNotNull(thread);
        checkNotNull(aesKey);
        ArrayList<Future<Boolean>> futures = new ArrayList<>();
        ExecutorService executorService = Executors.newFixedThreadPool(5);


        Members members = thread.getMembers();
        for (PID pid : members) {
            Future<Boolean> future = executorService.submit(() -> {
                try {
                    return connectPeer(context,
                            pid, aesKey, supportDiscovery, protectRelay, protectPeer, updateUser);
                } catch (Throwable e) {
                    Log.e(TAG, e.getLocalizedMessage(), e);
                    return false;
                }
            });
            futures.add(future);
        }
        return futures;
    }


    public static boolean connectMembers(@NonNull Context context,
                                         @NonNull Thread thread,
                                         @NonNull String aesKey,
                                         boolean supportDiscovery,
                                         boolean protectPeer,
                                         boolean protectRelay,
                                         boolean updateUser,
                                         boolean shortcut) {
        checkNotNull(context);
        checkNotNull(aesKey);
        checkNotNull(thread);

        boolean result = true;

        List<Future<Boolean>> futures = connectMembersAsync(
                context, thread, aesKey, supportDiscovery, protectPeer, protectRelay, updateUser);
        for (Future<Boolean> future : futures) {
            try {
                if (!future.get()) {
                    result = false;
                    if (shortcut) {
                        return result;
                    }
                }
            } catch (Throwable e) {
                Log.e(TAG, e.getLocalizedMessage(), e);
                result = false;
            }
        }
        return result;
    }

    public static boolean connectMembers(@NonNull Context context,
                                         @NonNull threads.core.api.Thread thread,
                                         @NonNull String aesKey,
                                         boolean supportDiscovery,
                                         boolean protectPeer,
                                         boolean protectRelay,
                                         boolean updateUser,
                                         int timeout,
                                         boolean shortcut) {
        checkNotNull(context);
        checkNotNull(thread);
        checkNotNull(aesKey);
        checkArgument(timeout > 0);
        boolean result = true;
        List<Future<Boolean>> futures = connectMembersAsync(
                context, thread, aesKey, supportDiscovery,
                protectPeer, protectRelay, updateUser, timeout);
        for (Future<Boolean> future : futures) {
            try {
                if (!future.get()) {
                    result = false;
                    if (shortcut) {
                        return result;
                    }
                }
            } catch (Throwable e) {
                Log.e(TAG, e.getLocalizedMessage(), e);
                result = false;
            }
        }
        return result;
    }
}

==> api-0.6.2-sources.jar: threads/core/ConnectService.java <==
package threads.core;

import android.content.Context;
import android.util.Log;

import androidx.annotation.NonNull;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;

import threads.core.api.Addresses;
import threads.core.api.Members;
import threads.core.api.MultiAddresses;
import threads.core.api.Note;
import threads.core.api.Thread;
import threads.ipfs.IPFS;
import threads.ipfs.api.PID;

import static androidx.core.util.Preconditions.checkArgument;
import static androidx.core.util.Preconditions.checkNotNull;

public class ConnectService {

    private static final String TAG = ConnectService.class.getSimpleName();

    public static boolean connectPeer(@NonNull Context context,
                                      @NonNull PID peer,
                                      @NonNull String aesKey,
                                      @NonNull String protectSwarm,
                                      boolean supportDiscovery,
                                      boolean updateUser) {
        checkNotNull(context);
        checkNotNull(aesKey);
        checkNotNull(peer);
        checkNotNull(protectSwarm);
        final int timeout = Preferences.getConnectionTimeout(context);
        return connectPeer(context, peer, aesKey, protectSwarm, timeout,
                supportDiscovery, updateUser);
    }

    public static boolean connectPeer(@NonNull Context context,
                                      @NonNull PID pid,
                                      @NonNull String aesKey,
                                      @NonNull String protectSwarm,
                                      int timeout,
                                      boolean supportDiscovery,
                                      boolean updateUser) {
        checkNotNull(context);
        checkNotNull(aesKey);
        checkNotNull(pid);
        checkNotNull(protectSwarm);
        checkArgument(timeout > 0);
        if (!Network.isConnected(context)) {
            return false;
        }


        if (supportDiscovery) {
            threads.core.api.PeerInfo peer = IdentityService.getPeerInfo(
                    context, pid, aesKey, updateUser);
            if (peer != null) {

                if (swarmConnect(context, peer, protectSwarm, timeout)) {
                    return true;
                }

            }

        }
        final IPFS ipfs = Singleton.getInstance(context).getIpfs();
        if (ipfs != null) {
            ipfs.swarmConnect(pid, timeout);

            return ipfs.isConnected(pid);

        }
        return false;
    }


    public static void swarmUnProtect(@NonNull Context context,
                                      @NonNull threads.core.api.PeerInfo peer,
                                      @NonNull String swarmProtect) {
        checkNotNull(context);
        checkNotNull(peer);
        checkNotNull(swarmProtect);
        final IPFS ipfs = Singleton.getInstance(context).getIpfs();
        if (ipfs != null) {
            Addresses addresses = peer.getAddresses();
            for (String relay : addresses.keySet()) {
                PID pid = PID.create(relay);
                if (!swarmProtect.isEmpty()) {
                    ipfs.unProtectPeer(pid, swarmProtect);
                }
            }
        }
    }

    public static boolean swarmConnect(@NonNull Context context,
                                       @NonNull threads.core.api.PeerInfo peer,
                                       @NonNull String protectSwarm,
                                       int timeout) {
        checkNotNull(context);
        checkNotNull(peer);
        checkArgument(timeout > 0);

        final Singleton.ConsoleListener consoleListener =
                Singleton.getInstance(context).getConsoleListener();

        final IPFS ipfs = Singleton.getInstance(context).getIpfs();
        if (ipfs != null) {
            Addresses addresses = peer.getAddresses();
            for (String relay : addresses.keySet()) {
                try {
                    String ma = addresses.get(relay);
                    checkNotNull(ma);
                    PID relayPID = PID.create(relay);
                    boolean relayConnected = ipfs.isConnected(relayPID);
                    if (!relayConnected) {
                        relayConnected = ipfs.swarmConnect(
                                ma + "/" + IPFS.Style.p2p.name() + "/" + relay,
                                timeout);
                    }
                    if (relayConnected) {

                        // be pessimistic and assume relay and no autonat and no pubsub
                        // anyway when it is, it they are the will be evaluated later
                        GatewayService.storePeer(context, PID.create(relay), ma,
                                true, false, false, 0);

                        if (!protectSwarm.isEmpty()) {
                            ipfs.protectPeer(PID.create(relay), protectSwarm);
                        }
                    }

                    if (relayConnected) {
                        String address = ipfs.relay_address(ma, relayPID, peer.getPID());
                        ipfs.swarmConnect(address, timeout);
                    }

                } catch (Throwable e) {
                    Log.e(TAG, e.getLocalizedMessage(), e);
                }
            }


            MultiAddresses multiAddresses = peer.getMultiAddresses();
            for (String address : multiAddresses) {

                try {
                    String ma = address.concat(
                            "/" + IPFS.Style.p2p + "/" + peer.getPID().getPid());
                    consoleListener.debug("Try connect : " + ma);


                    if (ipfs.isConnected(peer.getPID())) {
                        break;
                    } else {
                        boolean connect = ipfs.swarmConnect(ma, timeout);

                        if (connect) {
                            consoleListener.info("Success connect : " + ma);
                            break;
                        }
                    }
                } catch (Throwable e) {
                    Log.e(TAG, e.getLocalizedMessage(), e);
                }
            }


            return ipfs.isConnected(peer.getPID());
        }
        return false;
    }

    public static List<Future<Boolean>> connectMembersAsync(
            @NonNull Context context,
            @NonNull threads.core.api.Thread thread,
            @NonNull String aesKey,
            @NonNull String swarmProtect,
            boolean supportDiscovery,
            boolean updateUser,
            int timeout) {
        checkNotNull(context);
        checkNotNull(thread);
        checkNotNull(aesKey);
        checkNotNull(swarmProtect);
        checkArgument(timeout > 0);


        ArrayList<Future<Boolean>> futures = new ArrayList<>();
        ExecutorService executorService = Executors.newFixedThreadPool(5);
        Members members = thread.getMembers();
        for (PID pid : members) {
            Future<Boolean> future = executorService.submit(() -> {
                try {
                    return connectPeer(context, pid, aesKey, swarmProtect, timeout,
                            supportDiscovery, updateUser);
                } catch (Throwable e) {
                    Log.e(TAG, e.getLocalizedMessage(), e);
                    return false;
                }
            });
            futures.add(future);
        }
        return futures;
    }


    public static boolean publishNote(@NonNull Context context,
                                      @NonNull Note note,
                                      @NonNull String aesKey) {

        checkNotNull(context);
        checkNotNull(note);
        checkNotNull(aesKey);

        final THREADS threads = Singleton.getInstance(context).getThreads();
        final IPFS ipfs = Singleton.getInstance(context).getIpfs();
        Thread thread = threads.getThread(note);
        checkNotNull(thread);

        boolean notifyAll = true;
        Members members = thread.getMembers();
        for (PID pid : members) {

            try {

                boolean result = ipfs.isConnected(pid);
                if (result) {
                    result = threads.publishNote(ipfs, pid, note);
                }

                if (!result) {
                    notifyAll = false;
                }

            } catch (Throwable e) {
                Log.e(TAG, e.getLocalizedMessage(), e);
                return false;
            }

        }
        return notifyAll;
    }

    public static List<Future<Boolean>> connectMembersAsync(
            @NonNull Context context,
            @NonNull Thread thread,
            @NonNull String aesKey,
            @NonNull String swarmProtect,
            boolean supportDiscovery,
            boolean updateUser) {

        checkNotNull(context);
        checkNotNull(thread);
        checkNotNull(aesKey);
        checkNotNull(swarmProtect);
        ArrayList<Future<Boolean>> futures = new ArrayList<>();
        ExecutorService executorService = Executors.newFixedThreadPool(5);


        Members members = thread.getMembers();
        for (PID pid : members) {
            Future<Boolean> future = executorService.submit(() -> {
                try {
                    return connectPeer(context, pid, aesKey, swarmProtect,
                            supportDiscovery, updateUser);
                } catch (Throwable e) {
                    Log.e(TAG, e.getLocalizedMessage(), e);
                    return false;
                }
            });
            futures.add(future);
        }
        return futures;
    }


    public static boolean connectMembers(@NonNull Context context,
                                         @NonNull Thread thread,
                                         @NonNull String aesKey,
                                         @NonNull String swarmProtect,
                                         boolean supportDiscovery,
                                         boolean updateUser,
                                         boolean shortcut) {
        checkNotNull(context);
        checkNotNull(aesKey);
        checkNotNull(thread);
        checkNotNull(swarmProtect);
        boolean result = true;

        List<Future<Boolean>> futures = connectMembersAsync(
                context, thread, aesKey, swarmProtect, supportDiscovery, updateUser);
        for (Future<Boolean> future : futures) {
            try {
                if (!future.get()) {
                    result = false;
                    if (shortcut) {
                        return result;
                    }
                }
            } catch (Throwable e) {
                Log.e(TAG, e.getLocalizedMessage(), e);
                result = false;
            }
        }
        return result;
    }

    public static boolean connectMembers(@NonNull Context context,
                                         @NonNull threads.core.api.Thread thread,
                                         @NonNull String aesKey,
                                         @NonNull String swarmProtect,
                                         boolean supportDiscovery,
                                         boolean updateUser,
                                         int timeout,
                                         boolean shortcut) {
        checkNotNull(context);
        checkNotNull(thread);
        checkNotNull(aesKey);
        checkNotNull(swarmProtect);
        checkArgument(timeout > 0);
        boolean result = true;
        List<Future<Boolean>> futures = connectMembersAsync(
                context, thread, aesKey, swarmProtect, supportDiscovery,
                updateUser, timeout);
        for (Future<Boolean> future : futures) {
            try {
                if (!future.get()) {
                    result = false;
                    if (shortcut) {
                        return result;
                    }
                }
            } catch (Throwable e) {
                Log.e(TAG, e.getLocalizedMessage(), e);
                result = false;
            }
        }
        return result;
    }
}
